
/* Graphik-Routinen f�r LCD-Modul EA-DIP122-5NLED
 * und SED1520 Controller                                             
 * Copyright (C) 2001, 2002  Michael Sch�nzler
 */

#include "hardware.h"
#include <inttypes.h>
#include <avr/pgmspace.h>
#include <stdbool.h>

#define TRUE true
#define FALSE false

void init_lcd (void);

/* SED1520 Controller Primitives */
void display_on (void);
void display_off (void);
void write_register1 (uint8_t data);
void write_register2 (uint8_t data);
uint8_t read_display_data1 (void);
uint8_t read_display_data2 (void);
void write_display_data1 (uint8_t data);
void write_display_data2 (uint8_t data);
void write_display_data (uint8_t data, uint8_t col, uint8_t page,
			 uint8_t (*op)(uint8_t, uint8_t, uint8_t),
			 uint8_t mask);
void set_page (uint8_t page);
void set_column (uint8_t col);
void until_lcd_controller1_ready (void);
void until_lcd_controller2_ready (void);

#define DISPLAY_ON 175
#define DISPLAY_OFF 174
#define SET_PAGE 184
#define SET_COLUMN 0
#define SET_DISPLAY_START_LINE 192
#define DUTY_16 168
#define DUTY_32 169
#define RESET 226

#define WIDTH 122
#define HEIGHT 32
#define MAX_X WIDTH - 1
#define MAX_Y HEIGHT - 1
#define MAX_COL 60
#define MAX_PAGE 3

/* Graphic Primitives */
/*                    */

typedef struct 
{
  uint8_t width;
  uint8_t height;
  uint8_t pixel_data[];
} bitmap;

typedef struct
{
  uint8_t dwidth_x;
  uint8_t dwidth_y;
  uint8_t bbx_width;
  uint8_t bbx_height;
  int8_t bbx_xoff;
  int8_t bbx_yoff;
  uint8_t *pixel;
} glyph;

typedef struct {
  void (*glyph_getter)(char c, glyph *g);
  void (*char_writer)(char c);
  uint8_t *font_data;
} font_handler;

typedef struct {
  uint16_t font_size;
  uint8_t vid;
  uint8_t fid;
  uint8_t TOCoff;
  uint8_t ascii_min;
  uint8_t nAscii;
  uint8_t FBBx;
  uint8_t FBBy;
  uint8_t descent;
  uint8_t BSP;
  uint8_t CSP;
  uint8_t LSP;
  uint16_t *toc;
  uint8_t *glyphs;
} lcd_font;

void lcd_drawpixel (uint8_t x, uint8_t y);
void lcd_clearpixel (uint8_t x, uint8_t y);
void lcd_drawline(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);

void lcd_drawbitmap_P (uint8_t x, uint8_t y, const bitmap *b);
void lcd_drawbitmap (uint8_t x, uint8_t y, bitmap *b);
void lcd_drawpixeldata (uint8_t x1, uint8_t y1, 
			uint8_t dx, uint8_t dy, uint8_t *pixel_data,
			uint8_t (*operation)(uint8_t, uint8_t, uint8_t));
void lcd_drawpixeldata_P (uint8_t x1, uint8_t y1, 
			  uint8_t dx, uint8_t dy, uint8_t *pixel_data,
			  uint8_t (*operation)(uint8_t, uint8_t, uint8_t));
void lcd_drawpixeldata_generic (uint8_t x, uint8_t y,
				uint8_t dx, uint8_t dy,
				uint8_t *pixel_data,
				uint8_t (*operation)(uint8_t, uint8_t,
						     uint8_t),
				uint8_t (*pixel_getter)(uint8_t, uint8_t,
				                        uint8_t, uint8_t,
							uint8_t *));
void lcd_invert (uint8_t x, uint8_t y, uint8_t dx, uint8_t dy);
void lcd_cleardisplay (void);

uint8_t lcd_or (uint8_t a, uint8_t b, uint8_t mask);
uint8_t lcd_xor (uint8_t a, uint8_t b, uint8_t mask);
uint8_t lcd_and (uint8_t a, uint8_t b, uint8_t mask);
uint8_t lcd_replace (uint8_t a, uint8_t b, uint8_t mask);
uint8_t lcd_not (uint8_t a, uint8_t b, uint8_t mask);

void lcd_write_char (char c);
void lcd_putchar (char c);
void lcd_init_font_handler (lcd_font *font_desc);
void lcd_init_font_handler_small (lcd_font *font);
void lcd_set_cursor (uint8_t x, uint8_t y);

uint8_t lcd_cursor_x;
uint8_t lcd_cursor_y;
uint8_t (*lcd_drawing_mode)(uint8_t, uint8_t, uint8_t);
font_handler lcd_current_font_handler;
void (*write_char)(char);

#define min(x,y) ({ \
	const typeof(x) _x = (x);	\
	const typeof(y) _y = (y);	\
	(void) (&_x == &_y);		\
	_x < _y ? _x : _y; })

#define max(x,y) ({ \
	const typeof(x) _x = (x);	\
	const typeof(y) _y = (y);	\
	(void) (&_x == &_y);		\
	_x > _y ? _x : _y; })

void init_font5x8(void);
void init_font8x8(void);
void init_font7x13bold(void);
void init_font8x12(void);
void init_font8prop (void);

void setfont_helv_bold_8 (void);
void setfont_helv_bold_10 (void);
void setfont_helv_bold_12 (void);
void setfont_helv_bold_14 (void);
void setfont_helv_bold_18 (void);
void setfont_helv_bold_20 (void);
void setfont_helv_medium_8 (void);
void setfont_helv_medium_10 (void);
void setfont_helv_medium_12 (void);
void setfont_helv_medium_14 (void);
void setfont_helv_medium_18 (void);
void setfont_helv_medium_20 (void);
void setfont_times_bold_14 (void);
void setfont_micro (void);
void setfont_fixed_5x7 (void);
void setfont_fixed_5x8 (void);
void setfont_fixed_6x9 (void);
void setfont_fixed_6x10 (void);
void setfont_fixed_6x12 (void);
void setfont_fixed_6x13 (void);
void setfont_fixed_bold_6x13 (void);
void setfont_fixed_bold_7x13 (void);
void setfont_fixed_10x20 (void);

void setfont_lucida_bold_sans_10 (void);
void setfont_lucida_bold_sans_12 (void);
void setfont_lucida_bold_sans_14 (void);
void setfont_lucida_bold_sans_18 (void);
void setfont_lucida_bold_sans_24 (void);
void setfont_lucida_bold_sans_34 (void);
void setfont_nimbus_sans_bold_condensed_17 (void);

